@@ -1,6 +1,6 @@
 /*
-* Copyright (C) 2019 Tianjin KYLIN Information Technology Co., Ltd.
-*               2010-2016 LXQt team
+* Copyright (C) 2023, KylinSoft Co., Ltd.
+*               2010-2016 LXQt team.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -20,7 +20,6 @@
 #include "modulemanager.h"
 #include "ukuimodule.h"
 #include "idlewatcher.h"
-#include "ukuismserver.h"
 #include "ukuisessiondebug.h"
 #include "xdgautostart.h"
 #include "xdgdesktopfile.h"
@@ -38,6 +37,7 @@
 #include <QDBusPendingCall>
 #include <QDBusReply>
 #include <QDir>
+#include <QDBusReply>
 /* qt会将glib里的signals成员识别为宏，所以取消该宏
  * 后面如果用到signals时，使用Q_SIGNALS代替即可
  **/
@@ -48,12 +48,19 @@
 #define SESSION_REQUIRED_COMPONENTS "org.ukui.session.required-components"
 #define SESSION_REQUIRED_COMPONENTS_PATH "/org/ukui/desktop/session/required-components/"
 
-extern UKUISMServer*& getGlobalServer();
+#define SSWND_DBUS_SERVICE     "org.ukui.ScreenSaverWnd"
+#define SSWND_DBUS_PATH        "/"
+#define SSWND_DBUS_INTERFACE   "org.ukui.ScreenSaverWnd"
 
-std::map<QString, int> ModuleManager::m_startupMap = {};
+#define FD_DBUS_SERVICE     "org.freedesktop.DBus"
+#define FD_DBUS_PATH        "/org/freedesktop/DBus"
+#define FD_DBUS_INTERFACE   "org.freedesktop.DBus"
+
+std::map<const QString, int> ModuleManager::m_startupMap = {};
 
 void ModuleManager::playBootMusic(bool arg)
 {
+    if( arg ) return;
     //set default value of whether boot-music is opened
     bool play_music = true;
     if (QGSettings::isSchemaInstalled("org.ukui.session")) {
@@ -112,6 +119,8 @@ ModuleManager::ModuleManager( QObject* parent) : QObject(parent)
                                          QString("org.freedesktop.login1.Manager"),
                                          QString("PrepareForSleep"), this, SLOT(weakup(bool)));
     constructStartupList();
+
+    ChkScreenLockStartup();
 }
 
 void ModuleManager::weakup(bool arg)
@@ -359,6 +368,9 @@ void ModuleManager::startCompsite()
     if (isCompsiteStarted) return;
     isCompsiteStarted = true;
 
+    timerUpdate();
+
+/*    //按kwin要求，kwin现在启动后自动开混成
     // start composite
     QDBusInterface dbus("org.ukui.KWin", "/Compositor", "org.ukui.kwin.Compositing", QDBusConnection::sessionBus());
 
@@ -370,7 +382,7 @@ void ModuleManager::startCompsite()
         dbus.call("resume");
 
         timerUpdate();
-    }
+    }*/
 }
 
 void ModuleManager::startup()
@@ -383,7 +395,7 @@ void ModuleManager::startup()
             isDesktopStarted = true;
             connect(this, &ModuleManager::wmfinished, [=](){ startCompsite(); });
 
-            startProcess(mWindowManager, false);
+//            startProcess(mWindowManager, true);
             startModuleTimer(twm, 18);
         } else {
             timerUpdate();
@@ -401,7 +413,7 @@ void ModuleManager::startup()
         connect(this, &ModuleManager::wmfinished, [=](){ startCompsite(); });
         connect(this, &ModuleManager::desktopfinished, [=](){ startCompsite(); });
 
-        dostartwm();
+//        dostartwm();
     } else {
         QTimer::singleShot(500, this, SLOT(timerUpdate()));
     }
@@ -410,7 +422,7 @@ void ModuleManager::startup()
     for (XdgDesktopFileList::const_iterator i = mInitialization.constBegin(); i != mInitialization.constEnd(); ++i) {
         startProcess(*i, true);
     }
-    startModuleTimer(tusd,5);
+    startModuleTimer(twm,3);
 
     startProcess(mPanel, true);
     startModuleTimer(tpanel,5);
@@ -428,30 +440,22 @@ void ModuleManager::startup()
 
 void ModuleManager::dostartwm()
 {
-    if (mWindowManager.name() != "UKUI-KWin") {
-        qDebug() << "Start window manager: " << mWindowManager.name();
-        startProcess(mWindowManager, false);
+    qDebug() << "Start window manager: " << mWindowManager.name();
+    if (mWindowManager.name() == "UKUI-KWin") {
+        startProcess(mWindowManager, true);
         startModuleTimer(twm, 18);
     } else {
-        startProcess(mWindowManager, false);
+        startProcess(mWindowManager, true);
         isWMStarted = true;
     }
 }
 
 void ModuleManager::timerUpdate()
 {
-    playBootMusic(true);
     QTimer::singleShot(500, this, [&](){
         emit finished();
     });
 
-    //启动用户自定义的开机自启应用
-    auto appManagerConn = QDBusConnection::sessionBus().connect("com.kylin.AppManager",
-                                                               "/com/kylin/AppManager",
-                                                               "com.kylin.AppManager",
-                                                               "startSuccess",this,
-                                                               SLOT(openAppWithAppManager()));
-
     if (isWayland) {
         qDebug() << "Start desktop: ";
         for (XdgDesktopFileList::const_iterator i = mDesktop.constBegin(); i != mDesktop.constEnd(); ++i) {
@@ -480,19 +484,37 @@ void ModuleManager::timerUpdate()
         startProcess(*i, true);
     }
 
-//    for (XdgDesktopFileList::const_iterator i = m_userAutoStartApp.constBegin(); i != m_userAutoStartApp.constEnd(); ++i) {
-//        openAppWithAppManager(*i);
-//    }
+    //启动用户自定义的开机自启应用
+    for (XdgDesktopFileList::const_iterator i = m_userAutoStartApp.constBegin(); i != m_userAutoStartApp.constEnd(); ++i) {
+        qDebug() << "start user's autostart application " << i->fileName();
+        openAppWithAppManager(*i);
+    }
 
-//    //恢复会话
-//    if (QGSettings::isSchemaInstalled("org.ukui.session")) {
-//        QGSettings *gset = new QGSettings("org.ukui.session", "/org/ukui/desktop/session/", this);
-//        bool restoreSession = gset->get("restore-session").toBool();
-//        if (restoreSession) {
-//            qDebug(UKUI_SESSION) << "began restore session";
-//            getGlobalServer()->restoreSession();
-//        }
-//    }
+    if (QGSettings::isSchemaInstalled("org.ukui.session")) {
+        QGSettings *gset = new QGSettings("org.ukui.session", "/org/ukui/desktop/session/", this);
+        bool restoreSession = gset->get("restore-session").toBool();
+        if (restoreSession) {
+            //此处设置800ms延迟的原因是，当程序运行到这里时，已经启动了很多应用，根据测试，所有这些应用启动完并且完成与xmspserver的通信一般就需要800ms，如果这里不等待800ms，
+            //然后直接调用xsmpserver的restoreSession接口，则xsmpserver的主循环会被用来启动那些待恢复的应用,从而无暇顾及其他应用的信号请求，造成阻塞，影响整个启动流程
+            //之前觉得把xsmpserver分离出去可以解决这个问题，现在看来还是不行，因为ukuismserver还是要跟ukui-session交互并且判断是否该应用是否已经已经启动过，真正要解决这个
+            //问题，还是要把启动阶段划分好，在restoreSession阶段保证就不会有别的应用干扰。
+            QTimer::singleShot(800, [](){
+                //调用ukuismserver恢复会话的接口
+                qDebug(UKUI_SESSION) << "began restore session";
+                QDBusInterface ukuismserverFace("org.ukui.ukuismserver",
+                                                "/UKUISMServer",
+                                                "org.ukui.ukuismserver",
+                                                QDBusConnection::sessionBus());
+
+                if (!ukuismserverFace.isValid()) {
+                    qDebug() << "ukuisminterface not valid";
+                    return;
+                }
+
+                ukuismserverFace.call(QDBus::CallMode::NoBlock, "restoreSession");
+            });
+        }
+    }
 }
 
 void ModuleManager::startProcess(const XdgDesktopFile &file, bool required)
@@ -507,7 +529,7 @@ void ModuleManager::startProcess(const XdgDesktopFile &file, bool required)
     if (!mNameMap.contains(name)) {
         UkuiModule *proc = new UkuiModule(file, this);
         connect(proc, &UkuiModule::moduleStateChanged, this, &ModuleManager::moduleStateChanged);
-        proc->start();
+        proc->startUKUIModule();
 
         mNameMap[name] = proc;
 
@@ -540,6 +562,22 @@ void ModuleManager::stopProcess(const QString& name)
      }
 }
 
+bool ModuleManager::startProcess(const QString &name, const QStringList &args)
+{
+    if (isProgramStarted(std::move(name))) {
+        return false;
+    }
+
+    //这个接口供外部应用通过session打开应用使用，需要使用那个进程非正常死亡再拉起机制吗？
+    QProcess *proc = new QProcess(this);
+    proc->start(name, args);
+    proc->closeReadChannel(QProcess::StandardOutput);
+    proc->closeReadChannel(QProcess::StandardError);
+    m_startupMap[std::forward<const QString>(name)] = 1;
+
+    return true;
+}
+
 bool ModuleManager::nativeEventFilter(const QByteArray &eventType, void *message, long *result)
 {
     if (eventType != "xcb_generic_event_t") // We only want to handle XCB events
@@ -555,11 +593,16 @@ void ModuleManager::insertStartupList(QString &&str)
     }
 }
 
-bool ModuleManager::isProgramStarted(QString &&str)
+bool ModuleManager::isProgramStarted(const QString &&str)
 {
-    auto it = m_startupMap.find(std::forward<QString>(str));
+    //需要对str做处理，取出可执行程序名，str一般是一个完整的二进制路径，所以既需要判断二进制路径也要判断二进制名
+    QString appName = str.mid(str.lastIndexOf(QDir::separator()) + 1);
+    auto itFullPath = m_startupMap.find(str);
+    auto itName = m_startupMap.find(appName);
+    qDebug() << "judge if start before " << appName << " " << str;
 
-    if (it != m_startupMap.end()) {
+    if (itFullPath != m_startupMap.end() || itName != m_startupMap.end()) {
+        qDebug() << str << "has start up before";
         return true;
     }
 
@@ -572,23 +615,22 @@ bool ModuleManager::autoRestart(const XdgDesktopFile &file)
     return file.value(auto_restart).toBool();
 }
 
-bool ModuleManager::openAppWithAppManager()
+bool ModuleManager::openAppWithAppManager(const XdgDesktopFile &file)
 {
     QDBusInterface face("com.kylin.AppManager",
                         "/com/kylin/AppManager",
                         "com.kylin.AppManager",
                         QDBusConnection::sessionBus());
-
     if (!face.isValid()) {
-        qDebug() << "com.kylin.AppManager dose not exit";
+        qCDebug(UKUI_SESSION()) << "AppManager interface not valid";
         return false;
     }
 
-    for (XdgDesktopFileList::const_iterator i = m_userAutoStartApp.constBegin(); i != m_userAutoStartApp.constEnd(); ++i) {
-        QString name = i->fileName();
-        QVariant desktopFilePath(name);
-        QDBusReply<bool> reply = face.call("LaunchApp", desktopFilePath);
-        qDebug() << "LaunchApp " << name << " , and replay is " << reply;
+    QString name = file.fileName();
+    QVariant desktopFilePath(name);
+    QDBusReply<bool> reply = face.call("LaunchApp", desktopFilePath);
+    if (reply.isValid()) {
+        return reply.value();
     }
 
     return true;
@@ -596,7 +638,7 @@ bool ModuleManager::openAppWithAppManager()
 
 void ModuleManager::restartModules(int /*exitCode*/, QProcess::ExitStatus exitStatus)
 {
-    if (getGlobalServer()->prepareForShutdown()) {
+    if (isShutingDown) {
         qDebug() << "Logout phase, don't Restart";
         return;
     }
@@ -623,7 +665,7 @@ void ModuleManager::restartModules(int /*exitCode*/, QProcess::ExitStatus exitSt
             qDebug() << "Process" << procName << "(" << proc << ") exited correctly. " << "With the exitcode = " << proc->exitCode() << ",exitStatus = " << exitStatus;
         } else {
             qDebug() << "Process" << procName << "(" << proc << ") has to be restarted. " << "With the exitcode = " << proc->exitCode() << ",exitStatus = " << exitStatus;
-            proc->start();
+            proc->startUKUIModule();
             proc->restartNum++;
             return;
         }
@@ -632,6 +674,16 @@ void ModuleManager::restartModules(int /*exitCode*/, QProcess::ExitStatus exitSt
     proc->deleteLater();
 }
 
+bool ModuleManager::getIsShutingDown() const
+{
+    return isShutingDown;
+}
+
+void ModuleManager::setIsShutingDown(bool value)
+{
+    isShutingDown = value;
+}
+
 void ModuleManager::logout(bool doExit)
 {
     // /org/freedesktop/login1/session/self 和 /org/freedesktop/login1/session/auto
@@ -647,4 +699,58 @@ void ModuleManager::logout(bool doExit)
     }
 }
 
+void ModuleManager::ChkScreenLockStartup()
+{
+    QDBusInterface *iface = new QDBusInterface(SSWND_DBUS_SERVICE,
+                                               SSWND_DBUS_PATH,
+                                               SSWND_DBUS_INTERFACE,
+                                               QDBusConnection::sessionBus(),
+                                               this);
+    if (iface && iface->isValid()) {
+        QDBusReply<bool> reply =  iface->call("IsStartupMode");
+        if (reply.isValid()) {
+            qDebug()<<"ScreenLockStartup:"<<reply.value();
+            isScreenLockStartup = reply.value();
+        }
+        //connect(iface, SIGNAL(StartupModeChanged(bool)), this, SLOT(onSLStartupModeChanged(bool)));
+    }
+    // 监听锁屏退出
+    if (isScreenLockStartup) {
+        QDBusInterface *dbusService = new QDBusInterface(FD_DBUS_SERVICE,
+                                         FD_DBUS_PATH,
+                                         FD_DBUS_INTERFACE,
+                                         QDBusConnection::sessionBus(),
+                                         this);
+        connect(dbusService, SIGNAL(NameOwnerChanged(QString, QString, QString)),
+                this, SLOT(onServiceStatusChanged(QString, QString, QString)));
+    }
+}
+
+void ModuleManager::onServiceStatusChanged(const QString &name,
+                                           const QString &oldOwner,
+                                           const QString &newOwner)
+{
+    if(name == SSWND_DBUS_SERVICE)
+    {
+        bool isScreenLockStarted = !newOwner.isEmpty();
+        qDebug() << "ScreenLock service status changed:"
+                 << (isScreenLockStarted ? "activate" : "inactivate");
+        if (!isScreenLockStarted) {
+            onSLStartupModeChanged(isScreenLockStarted);
+        }
+    }
+}
+
+void ModuleManager::onSLStartupModeChanged(bool isStartup)
+{
+    static bool isPlayBootMusicDone = false;
+    isScreenLockStartup = isStartup;
+    qDebug()<<"onSLStartupModeChanged:"<<isScreenLockStartup;
+    if (!isScreenLockStartup && !isPlayBootMusicDone) {
+        isPlayBootMusicDone = true;
+        playBootMusic(true);
+    }
+}
+
+
 #include "modulemanager.moc"
