@@ -1,6 +1,6 @@
 /* BEGIN_COMMON_COPYRIGHT_HEADER
- * (c)LGPL2+
- * Copyright: 2019 Tianjin KYLIN Information Technology Co., Ltd.
+ *
+ * Copyright (C) 2023, KylinSoft Co., Ltd.
  *
  * This program or library is free software; you can redistribute it
  * and/or modify it under the terms of the GNU Lesser General Public
@@ -31,7 +31,7 @@ class UkuiModule : public QProcess
     Q_OBJECT
 public:
     UkuiModule(const XdgDesktopFile& file, QObject *parent = nullptr);
-    void start();
+    void startUKUIModule();
     void terminate();
     bool isTerminating();
 
@@ -40,9 +40,6 @@ public:
     const XdgDesktopFile file;
     const QString fileName;
 
-private:
-    void startWM(QString &command, QStringList &args);
-
 Q_SIGNALS:
     void moduleStateChanged(QString name, bool state);
 
